function compareDblInt2(input_mode)
%% Compare permissive and aggressive control schemes
% for permissive control scheme, set input_mode to 'p'
% for aggressive control scheme, set input_mode to 'a'
%
% input_mode: Single character string, either 'a' or 'p' to denote either
% aggressive or permissive input selection.  Optional.  Default = 'p'.
%
% Mo Chen, 2012-04-05

% clear all
run('addPathToKernel');

if(nargin < 1)
  input_mode = 'p';
end

dt = 0.3;
iterations = 10;

% Generate reach set
[g ctrl inev unsafe] = varyingDoubleIntegratorSampled(dt,iterations,'medium');
% Generate controls
num_input_samples = 7;
[ aggressive permissive g_ds ] = varyingDoubleIntegratorControl(g, ctrl, inev, unsafe, num_input_samples, 1);

iterations = 10;

% Store controls into 3D array
num_input_samples = 3;
input_samples = linspace(-1, +1, num_input_samples);
ulist = input_samples; % This is used later to generate trajectories

input_samples_array_a = repmat(reshape(input_samples, [ 1, 1, num_input_samples ]), [ g.shape, 1 ]);
input_samples_array_p = repmat(reshape(input_samples, [ 1, 1, num_input_samples ]), [ g.shape, 1 ]);

% Set any invalid inputs to be NaN.
input_samples_array_a(~aggressive) = nan;
input_samples_array_p(~permissive) = nan;

discReach = inev;
discDonut = ctrl;

plotType = 'contour';


%% Generate sample trajectories
% Initial conditions
maxNumTraj = 5;
numTraj = 3 * maxNumTraj; % number of initial conditions
x01 = -5 + 10*rand(numTraj,1);
x02 = -1.5 + 3 *rand(numTraj,1);

% numTraj = 1;
% x01 = -0.5;
% x02 = 0.5;

% figure for sample trajectories
comp2 = figure;
figure(comp2)
hold on
c2 = visualizeLevelSet(g, discReach, plotType, 0);
set(c2,'lineWidth',1, 'lineColor','b')
c2a = visualizeLevelSet(g, discDonut, plotType, 0);
set(c2a,'lineWidth',1, 'lineColor','r')

% plot(x01,x02,'k.') % Plot initial conditions

global u
global y0

% Figure for trajectory values
valuefig = figure;

plottedTraj = 1;
for i = 1:length(x01)
    % Make sure initial condition is in the safe set
    valueReach = interpn(g.xs{1},g.xs{2},discReach,x01(i),x02(i));
    
    % Vectors to store a trajectory
    tt = [0]; x1 = [x01(i)]; x2 = [x02(i)]; uu = [];
    tSample = [0];
    tSampleValues = [interpn(g.xs{1},g.xs{2},discReach,x01(i),x02(i))];
    
    % Decision variable for plotting trajectories
    plotOrNot = 0;
    if valueReach > 0
        y0 = [x01(i) x02(i)];   % Initial point
        initialPt=y0;
        
        for k = 1:iterations            % iterations is from dblIntSemiL.m
            % Check if trajectory is inside or outside donut
            valueDonut = interpn(g.xs{1},g.xs{2},discDonut,initialPt(1),initialPt(2));
            
            % Plot initial reach set function value
            
            % Run trajectory for one time step
            if valueDonut > 0
                if valueDonut > 0.7
                    % drive state to donut if far from donut
                    lastValSegment = 1e9;
                    for l = 1:length(ulist) % pick the best control from list of safe controls
                        u = ulist(l);
                        [tlist,segmentlist]=ode45(@dblIntfunu, [0 dt], initialPt);
                        valSegment = interpn(g.xs{1},g.xs{2},discReach,segmentlist(:,1),segmentlist(:,2));
                        if valSegment(end) < lastValSegment
                            t = tlist;
                            segment = segmentlist;
                            ubar=u;
                            lastValSegment = valSegment(end);
                        end
                    end                    
                else
                    % random input if outside and close to donut
                    randCtrl = -1 + 2*rand();
                    u = randCtrl;
                    [t,segment]=ode45(@dblIntfunu, [0 dt], initialPt);
                    ubar = u;
                end
            else
                
                % u* as input if inside donut
                switch input_mode
                    case 'a'
                        lastValSegment = -1e9;
                    case 'p'
                        lastValSegment = 1e9;
                    otherwise
                        error('Unknown input mode: %s', input_mode);
                end    
                            
                for l = 1:length(ulist) % pick the best control from list of safe controls
                    u = ulist(l);
                    [tlist,segmentlist]=ode45(@dblIntfunu, [0 dt], initialPt);
                    valSegment = interpn(g.xs{1},g.xs{2},discReach,segmentlist(:,1),segmentlist(:,2));
                    switch input_mode
                        case 'a'
                            if valSegment(end) > lastValSegment
                                t = tlist;
                                segment = segmentlist;
                                ubar=u;
                                lastValSegment = valSegment(end);
                            end
                            plotOrNot = 1;      % Enable plotting for this trajectory
%                           disp(ulist)
                        case 'p'
                            lowerVal = interpn(g.xs{1},g.xs{2},discReach,x1(end),x2(end));

                            if (valSegment(end) < lastValSegment) && (valSegment(end) >= lowerVal)
                                t = tlist;
                                segment = segmentlist;
                                ubar=u;
                                lastValSegment = min(valSegment);
                            end
                            plotOrNot = 1;      % Enable plotting for this trajectory                            
%                           disp(ulist)
                    end
                end
                
            end
            
            % Store trajectory
            tt = [tt; t+(k-1)*dt];
            x1 = [x1; segment(:,1)];
            x2 = [x2; segment(:,2)];
            uu = [uu; ubar*ones(length(segment),1)];
            
            tSample = [tSample; tt(end)];
            tSampleValues = [tSampleValues; interpn(g.xs{1},g.xs{2},discReach,x1(end),x2(end))];
            
            % Update initial point
            initialPt = [x1(end) x2(end)];
        end
        
        if (plotOrNot > 0)
          
            % Plot trajectory on reach set figure
            figure(comp2)
            hold on
            x11 = x1;
            x22 = x2;
            plot(x1(1),x2(1),'k.')
            plot(x1,x2,'k-','linewidth',1)

            % Plot function values of trajectories
            trajValues = interpn(g.xs{1},g.xs{2},discReach,x1,x2);

            figure(valuefig)

            hold on
            plot(tt,trajValues,'b-','linewidth',1)
            plot(tSample, tSampleValues, 'ro')
%             plot(tt,uu+1,'g-')
%             plot(tt,x1,'g-')
%             plot(tt,x2,'g--')

            % Indicate which part of trajectory is inside donut
            donutValues = interpn(g.xs{1},g.xs{2},discDonut,x1,x2);
            plot(tt(donutValues<=0), trajValues(donutValues<=0),'b.')
%             break; % terminate after one trajectory is plotted

            % Record that we plotted a trajectory.
            plottedTraj = plottedTraj + 1;
            % Stop looping if we have plotted enough trajectories.
            if(plottedTraj > maxNumTraj)
              break;
            end
        end
    end
end

figure(comp2)
switch input_mode
    case 'a'
        title('Aggressive control')
    case 'p'
        title('Permissive control')
end
legend('Reach set','Donut set','Initial points','Trajectories')
xlabel('x_1'); ylabel('x_2')

figure(valuefig)
xlabel('t'); ylabel('function value')

end


%-----------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-----------------------------------------------------------------------
function dy = dblIntfunu(t,y)
dy = zeros(2,1);
dy(1) = y(2);
global u
dy(2) = cos(2*pi*y(1))*u;
end

